#include "StdAfx.h"
#include "AddLineForm.h"
#include "Codec.h"
#include "MainForm.h"

using namespace VaxTeleConfigSIP;

#define IN_BOUND_CALL_REJECT			0
#define IN_BOUND_CALL_CONNECT_USER		1
#define IN_BOUND_CALL_PUT_IN_QUEUE		2

bool AddLineForm::AddLineChecks(String^ sInCallTo, String^ sInCallType)
{
    if(ComboLine->Text->Length == 0)
    {
        MessageBox::Show("Please enter Line Name.", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
        return false;
    }

    if(ListBoxEnableCodec->Items->Count == 0)
    {
        MessageBox::Show("Please select a codec.", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
        return false;
    }

    sInCallType = "0";

    if(RadioConnectToUser->Checked == true)
    {
        if(ComboConnectToUser->SelectedIndex == -1)
        {
            MessageBox::Show("Please select the user/login to forward the incoming call.", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
            return false;
        }

        sInCallType = "1";
        sInCallTo = ComboConnectToUser->SelectedItem->ToString();
    }
    else if(RadioPutInQueue->Checked == true)
    {
        if(ComboPutInQueue->SelectedIndex == -1)
        {
            MessageBox::Show("Please select the queue to put the incoming call.", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
            return false;
        }

        sInCallType = "2";
        sInCallTo = ComboPutInQueue->SelectedItem->ToString();
    }

    return true;
}

void AddLineForm::BtnAddLine_Click(System::Object^  sender, System::EventArgs^  e)
{
	String^ sInCallTo = gcnew String("");
	String^ sInCallType = gcnew String("");

	if(AddLineChecks(sInCallTo, sInCallType) == false)
		return;

	if(ComboLine->FindStringExact(ComboLine->Text) != -1)
	{
		if(MessageBox::Show("Line already exist, do you want to update it?", "VaxTeleSIP", MessageBoxButtons::YesNo, MessageBoxIcon::Exclamation) == ::DialogResult::No)
			return;
	}

	ArrayList^ objParamList = gcnew ArrayList();

	objParamList->Add(ComboLine->Text);

	objParamList->Add(EditDisplayName->Text);
	objParamList->Add(EditUserName->Text);
	objParamList->Add(EditAuthUser->Text);
	objParamList->Add(EditPassword->Text);
	objParamList->Add(EditDomainRealm->Text);
	objParamList->Add(EditProxySIP->Text);
	objParamList->Add(EditOutboundProxySIP->Text);

	cCodec^ objCodec = gcnew cCodec();
	String^ sCodec = gcnew String("");

	for(int nCount = 0; nCount < ListBoxEnableCodec->Items->Count; nCount++)
		sCodec += objCodec->GetCodecNo((String^) ListBoxEnableCodec->Items[nCount]);

	objParamList->Add(sCodec);

	if(CheckBoxRegLine->Checked == true)
		objParamList->Add("REGISTER-YES");
	else
		objParamList->Add("REGISTER-NO");

	objParamList->Add(sInCallType);
	objParamList->Add(sInCallTo);

	MainForm::m_objMainForm->SendCommand("AddLine", objParamList);
}

void AddLineForm::AddLineForm_Load(System::Object^  sender, System::EventArgs^  e)
{
	MainForm::m_objMainForm->SendCommand("ShowLines");
	ClearDlgContent();
}

void AddLineForm::ClearDlgContent()
{
	ComboLine->Text = "";
	
	EditUserName->Text = "";
	EditAuthUser->Text = "";
	EditPassword->Text = "";
	EditDisplayName->Text = "";
	EditDomainRealm->Text = "";
	EditProxySIP->Text = "";
	EditOutboundProxySIP->Text = "";

	if(ComboConnectToUser->Items->Count > 0)
		ComboConnectToUser->SelectedIndex = 0;

	if(ComboPutInQueue->Items->Count > 0)
		ComboPutInQueue->SelectedIndex = 0;
            
	CheckBoxRegLine->Checked = false;
	RadioRejectCall->Checked = true;

	ListBoxEnableCodec->Items->Clear();
	ListBoxDisableCodec->Items->Clear();

	ListBoxEnableCodec->Items->Add(G711U_TEXT);
	ListBoxEnableCodec->Items->Add(G711A_TEXT);
	ListBoxEnableCodec->Items->Add(ILBC_TEXT);
	ListBoxEnableCodec->Items->Add(G610_TEXT);
	ListBoxEnableCodec->Items->Add(G729_TEXT);
}


void AddLineForm::BtnRemoveLine_Click(System::Object^  sender, System::EventArgs^  e)
{
	if(ComboLine->Text->Length == 0)
	{
		MessageBox::Show("Please enter line name.", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
		return;
	}

	ArrayList^ objParamList = gcnew ArrayList();
	objParamList->Add(ComboLine->Text);

	MainForm::m_objMainForm->SendCommand("RemoveLine", objParamList);
}

void AddLineForm::UpdateDlgContent(String^ sLineName, String^ sDisplayName, String^ sUserName, String^ sAuthUser, String^ sAuthPwd, String^ sRealm, String^ sProxy, String^ sOutboundProxy, String^ sCodecList, String^ sRegister, String^ sInCallType, String^ sInCallTo)
{
    if(ComboLine->Text != sLineName)
        return;
    
    EditDisplayName->Text = sDisplayName;
    EditUserName->Text = sUserName;

    EditAuthUser->Text = sAuthUser;
    EditPassword->Text = sAuthPwd;
    EditDomainRealm->Text = sRealm;
    
    EditProxySIP->Text = sProxy;
    EditOutboundProxySIP->Text = sOutboundProxy;

    if(sRegister == "REGISTER-YES")
        CheckBoxRegLine->Checked = true;
    else
        CheckBoxRegLine->Checked = false;
    
	UpdateIncomingCallSection(Convert::ToInt32(sInCallType), sInCallTo);

    ListBoxEnableCodec->Items->Clear();
    ListBoxDisableCodec->Items->Clear();
                
    cCodec^ objCodec = gcnew cCodec();

    for(int nCount = 0; nCount < TOTAL_SIZE_CODEC; nCount++)
		ListBoxDisableCodec->Items->Add(objCodec->GetCodecName(Convert::ToString(nCount)));
    
    String^ sCodecName;

    for (int nCount = 0; nCount < sCodecList->Length; nCount++)
    {
        sCodecName = objCodec->GetCodecName(sCodecList[nCount].ToString());
        ListBoxEnableCodec->Items->Add(sCodecName);
        ListBoxDisableCodec->Items->Remove(sCodecName);
    }
}

void AddLineForm::UpdateIncomingCallSection(int nInCallType, String^ sInCallTo)
{
    if(nInCallType == IN_BOUND_CALL_REJECT)
        RadioRejectCall->Checked = true;

    else if(nInCallType == IN_BOUND_CALL_CONNECT_USER)
	{
		int nIndex = ComboConnectToUser->FindStringExact(sInCallTo);

		if (nIndex == -1)
		{
			RadioRejectCall->Checked = true;
			return;
		}
	    
		if(ComboConnectToUser->Items->Count > 0)
			ComboConnectToUser->SelectedIndex = nIndex;
	    
		RadioConnectToUser->Checked = true;
	}
	else if(nInCallType == IN_BOUND_CALL_PUT_IN_QUEUE)
	{
		int nIndex = ComboPutInQueue->FindStringExact(sInCallTo);

		if (nIndex == -1)
		{
			RadioRejectCall->Checked = true;
			return;
		}

		if(ComboPutInQueue->Items->Count > 0)
			ComboPutInQueue->SelectedIndex = nIndex;
	                    
		RadioPutInQueue->Checked = true;
	}
}

void AddLineForm::ComboLine_SelectedIndexChanged(System::Object^  sender, System::EventArgs^  e)
{
	ArrayList^ objParamList = gcnew ArrayList();
	objParamList->Add(ComboLine->SelectedItem->ToString());

	MainForm::m_objMainForm->SendCommand("LineInfo", objParamList);
}

void AddLineForm::BtnClearLineDlg_Click(System::Object^  sender, System::EventArgs^  e)
{
	ClearDlgContent();
}

void AddLineForm::BtnDisableCodec_Click(System::Object^  sender, System::EventArgs^  e)
{
	if(ListBoxEnableCodec->SelectedItems->Count == 0) return;

    ListBoxDisableCodec->Items->Add(ListBoxEnableCodec->SelectedItem->ToString());
	ListBoxEnableCodec->Items->Remove(ListBoxEnableCodec->SelectedItem);
}

void AddLineForm::BtnEnableCodec_Click(System::Object^  sender, System::EventArgs^  e)
{
	if(ListBoxDisableCodec->SelectedItems->Count == 0) return;

	ListBoxEnableCodec->Items->Add(ListBoxDisableCodec->SelectedItem->ToString());
	ListBoxDisableCodec->Items->Remove(ListBoxDisableCodec->SelectedItem);
}

void AddLineForm::OnTelnetCommand(String^ sCommand, ArrayList^ objParamList)
{
    if (String::Compare(sCommand, "AddUser", true) == 0 && objParamList->Count == 1)
    {
        ComboConnectToUser->Items->Add(objParamList[0]);
        return;
    }

    if(String::Compare(sCommand, "RemoveUser", true) == 0 && objParamList->Count == 1)
    {
        int nIndex = ComboConnectToUser->FindStringExact((String^) objParamList[0]);
        if(nIndex == -1) return;
        
        ComboConnectToUser->Items->RemoveAt(nIndex);
        return;
    }

    if(String::Compare(sCommand, "AddQueue", true) == 0 && objParamList->Count == 1)
    {
        ComboPutInQueue->Items->Add(objParamList[0]);

        return;
    }

    if(String::Compare(sCommand, "RemoveQueue", true) == 0 && objParamList->Count == 1)
    {
        int nIndex = ComboPutInQueue->FindStringExact((String^) objParamList[0]);
        if(nIndex == -1) return;
        
        ComboPutInQueue->Items->RemoveAt(nIndex);

        return;
    }

    if(String::Compare(sCommand, "ShowQueues", true) == 0)
    {
        for(int nCount = 0; nCount < objParamList->Count; nCount++)
            ComboPutInQueue->Items->Add(objParamList[nCount]);
        
        if(ComboPutInQueue->Items->Count > 0)
            ComboPutInQueue->SelectedIndex = 0;
        
        return;
    }

    if(String::Compare(sCommand, "ShowUsers", true) == 0)
    {
        for(int nCount = 0; nCount < objParamList->Count; nCount++)
            ComboConnectToUser->Items->Add(objParamList[nCount]);
        
        if(ComboConnectToUser->Items->Count > 0)
            ComboConnectToUser->SelectedIndex = 0;
        
        return;
    }

    if(String::Compare(sCommand, "ShowLines", true) == 0)
    {
        for(int nCount = 0; nCount < objParamList->Count; nCount++)
            ComboLine->Items->Add(objParamList[nCount]);
        
        if(ComboLine->Items->Count > 0)
            ComboLine->SelectedIndex = 0;
        
        return;
    }

    if(String::Compare(sCommand, "LineInfo", true) == 0 && objParamList->Count == 12)
    {
        String^ sLineName = (String^) objParamList[0];
        String^ sDisplayName = (String^) objParamList[1];
        String^ sUserName = (String^) objParamList[2];
        String^ sAuthUser = (String^) objParamList[3];
        String^ sAuthPwd = (String^) objParamList[4];
        String^ sRealm = (String^) objParamList[5];
        String^ sProxy = (String^) objParamList[6];
        String^ sOutboundProxy = (String^) objParamList[7];

        String^ sCodecList = (String^) objParamList[8];

        String^ sRegister = (String^) objParamList[9];
        String^ sInCallType = (String^) objParamList[10];
        String^ sInCallTo = (String^) objParamList[11];

        UpdateDlgContent(sLineName, sDisplayName, sUserName, sAuthUser, sAuthPwd, sRealm, sProxy, sOutboundProxy, sCodecList, sRegister, sInCallType, sInCallTo);
        return;
    }

    if(String::Compare(sCommand, "RemoveLine", true) == 0 && objParamList->Count == 1)
    {
        int nIndex = ComboLine->FindStringExact((String^) objParamList[0]);
        if(nIndex == -1) return;

        ComboLine->Items->RemoveAt(nIndex);

        MessageBox::Show("Line removed successfully!", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
        ClearDlgContent();

        return;
    }

    if(String::Compare(sCommand, "AddLine", true) == 0 && objParamList->Count == 1)
    {
        ComboLine->Items->Add(objParamList[0]);
        MessageBox::Show("Line created successfully", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);

        return;
    }

    if(String::Compare(sCommand, "UpdateLine", true) == 0 && objParamList->Count == 1)
    {
        MessageBox::Show("Line updated successfully", "VaxTeleSIP", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
        return;
    }
}
