// TelnetClient.cpp : implementation file
//

#include "stdafx.h"
#include "VaxTeleConfigSIP.h"
#include "VaxTeleConfigDlg.h"
#include "TelnetClient.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define AUTH_LOGIN		100
#define AUTH_PASSWORD	101
#define AUTH_WAIT		102
#define AUTH_DONE		103

#define AUTH_STATUS_SUCCESS	103
#define AUTH_STATUS_FAIL	104
#define AUTH_STATUS_WAIT	105

/////////////////////////////////////////////////////////////////////////////
// CTelnetClient

CTelnetClient::CTelnetClient()
{
	AfxSocketInit();

	m_nAuthStep   = AUTH_LOGIN;
	m_nAuthStatus = AUTH_STATUS_WAIT;
}

CTelnetClient::~CTelnetClient()
{
}


// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CTelnetClient, CAsyncSocket)
	//{{AFX_MSG_MAP(CTelnetClient)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

/////////////////////////////////////////////////////////////////////////////
// CTelnetClient member functions

void CTelnetClient::Disconnect()
{
	SendCommand("bye", NULL);
	CAsyncSocket::Close();
}

BOOL CTelnetClient::Connect(LPCTSTR sToIP, UINT nToPort, LPCTSTR sLogin, LPCTSTR sPassword)
{
	CAsyncSocket::Close();
	
	if(!CAsyncSocket::Create())
		::MessageBox(NULL, "Unable to create windows socket", "Socket", MB_OK|MB_ICONEXCLAMATION);
		
	CAsyncSocket::Connect(sToIP, nToPort);
		
	m_sLoginTelnet    = sLogin;
	m_sPasswordTelnet = sPassword;

	m_nAuthStatus = AUTH_STATUS_WAIT;
	
	for(int nCount = 0; nCount < 40; nCount++)
	{
		ReceiveData();
		::Sleep(100);

		if(m_nAuthStatus != AUTH_STATUS_WAIT) 
			break;
	}

	if(m_nAuthStatus != AUTH_STATUS_SUCCESS)
	{
		::MessageBox(NULL, "Fail to connect to server.", "Socket", MB_OK|MB_ICONEXCLAMATION);
		return FALSE;	
	}
	
	::MessageBox(NULL, "Connected successfully.", "Socket", MB_OK|MB_ICONEXCLAMATION);
	return TRUE;
}

CString CTelnetClient::FindRequest()
{
	int nStart = m_sRecvBuffer.Find("\r\n");
	if(nStart == -1) return "";
	
	CString sCommand = m_sRecvBuffer.Left(nStart);
	m_sRecvBuffer.Delete(0, nStart + 2);

	return sCommand;
}

void CTelnetClient::FillRecvBuffer()
{
	while(TRUE)
	{
		char sBuffer[2048] = "\0";
		int nSize = Receive(sBuffer, 2048);

		if(nSize == SOCKET_ERROR || nSize == 0)
			break;
	
		m_sRecvBuffer += sBuffer;
		m_sRecvBuffer.Replace("VaxTele:>", "");
	}
}

BOOL CTelnetClient::ParseRequest(CString objRequest, CString& objCommand, CStringList* pParamList)
{
	int nEnd = objRequest.Find(" ");
	
	if(nEnd == -1) 
	{
		objCommand = objRequest;
		return TRUE;
	}

	objCommand = objRequest.Left(nEnd);
	objRequest.Delete(0, nEnd + 1);

	if(objCommand.CompareNoCase("MSG") == 0 || objCommand.CompareNoCase("StatusServerSIP") == 0 ||
	   objCommand.CompareNoCase("ErrorLog") == 0 || objCommand.CompareNoCase("StatusLog") == 0)
	{
		pParamList->AddTail(objRequest);
		return TRUE;
	}
		
	while(TRUE)
	{
		nEnd = objRequest.Find(" ");
		
		if(nEnd == -1) 
		{	
			pParamList->AddTail(objRequest);
			break;
		}
		
		pParamList->AddTail(objRequest.Left(nEnd));
		objRequest.Delete(0, nEnd + 1);
	}

	return TRUE;
}

void CTelnetClient::ReceiveData() 
{
	FillRecvBuffer();
	
	while(TRUE)
	{
		if(m_nAuthStep != AUTH_DONE)
		{
			OnAuthrization(m_sRecvBuffer);
			break;
		}
		
		CString objRequest = FindRequest();
		if(objRequest.IsEmpty()) break;
		
		CString objCommand;
		CStringList objParamList;

		if(!ParseRequest(objRequest, objCommand, &objParamList)) 
			break;

		OnRequest(objCommand, &objParamList);
	}
}

void CTelnetClient::OnAuthrization(CString sData)
{
	if(m_nAuthStep == AUTH_LOGIN)
	{
		if(sData.Find("Login:") == -1)
			return;
		
		m_nAuthStep = AUTH_PASSWORD;
		SendCommand(m_sLoginTelnet, NULL);

		m_sRecvBuffer = "";
	}
	else if(m_nAuthStep == AUTH_PASSWORD)
	{
		if(sData.Find("Password:") == -1)
			return;

		m_nAuthStep = AUTH_WAIT;
		SendCommand(m_sPasswordTelnet, NULL);

		m_sRecvBuffer = "";
	}
	else if(m_nAuthStep == AUTH_WAIT && sData.IsEmpty() == FALSE)
	{
		m_sRecvBuffer = "";
		
		if(sData.Find("Success", 0) == -1)
		{
			m_nAuthStep   = AUTH_LOGIN;
			m_nAuthStatus = AUTH_STATUS_FAIL;

			return;
		}

		m_nAuthStep   = AUTH_DONE;
		m_nAuthStatus = AUTH_STATUS_SUCCESS;
	}
}


void CTelnetClient::OnReceive(int nErrorCode) 
{
	ReceiveData();
	CAsyncSocket::OnReceive(nErrorCode);
}


void CTelnetClient::OnRequest(LPCTSTR sCommand, CStringList* pParamList)
{

	if(::lstrcmpi(sCommand, "BYE") == 0)
	{
		OnDisconnect();
		return;
	}


	CVaxTeleConfigDlg::m_pVaxTeleConfigDlg->OnTelnetCommand(sCommand, pParamList);
}


void CTelnetClient::SendCommand(LPCTSTR sCommand, CStringList* pParamList)
{
	CString sToData = sCommand;

	if(pParamList != NULL)
	{
		for(int nCount = 0; nCount < pParamList->GetCount(); nCount++)
		{
			POSITION pos   = pParamList->FindIndex(nCount);
			CString sParam = pParamList->GetAt(pos);

			sToData =  sToData + " " + sParam;
		}
	}
	
	sToData += "\r\n";

	CAsyncSocket::Send(sToData.GetBuffer(0), sToData.GetLength());
}

void CTelnetClient::OnDisconnect()
{
	Close();
	::MessageBox(NULL, "Connection to host lost.", "Socket", MB_OK|MB_ICONEXCLAMATION);	
}

void CTelnetClient::OnClose(int nErrorCode) 
{
	CAsyncSocket::OnClose(nErrorCode);
	OnDisconnect();
}
