// LineList.cpp: implementation of the CLineList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ServerSIP.h"
#include "LineList.h"
#include "Registry.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CLineList::CLineList()
{
	m_pLineListTEXT = new CMapLineStrToPtr;
	m_pLineListINT  = new CMapLineULongToPtr;

	m_lLineIdCount = 0;
}

CLineList::~CLineList()
{
	UnLoadLines();

	delete m_pLineListTEXT;
	delete m_pLineListINT;
}

LONG CLineList::ChooseLineId()
{
	while(TRUE)
	{
		m_lLineIdCount++;
	
		if(FindLine(m_lLineIdCount) == NULL) break;
	}

	return m_lLineIdCount;
}

void CLineList::UnLoadLines()
{
	CString sKey;
	CLineData* pLineData;
			
	int nTotalCall = m_pLineListTEXT->GetCount();
	
	CString* pLineNameList = new CString[nTotalCall];
	POSITION pos = m_pLineListTEXT->GetStartPosition();

	for(int nCount = 0; pos != NULL; nCount++)
	{
		m_pLineListTEXT->GetNextAssoc(pos, sKey, pLineData);
		pLineNameList[nCount] = sKey;
	}

	for(nCount = 0; nCount < nTotalCall; nCount++)
	{
		Remove(pLineNameList[nCount]);
	}
		
	m_pLineListINT->RemoveAll();
	m_pLineListTEXT->RemoveAll();

	delete[] pLineNameList;
}


void CLineList::LoadLines()
{
	UnLoadLines();
	
	CRegistry objReg;

	CStringList objLineList;
	objReg.EnumLine(&objLineList);

	CLineReg objLineReg;
	
	while(!objLineList.IsEmpty())
	{
		CString objLineName = objLineList.RemoveHead();
		
		if(!objReg.GetLine(objLineName, &objLineReg))
		{
			CServerSIP::m_pServerSIP->LogError("Unable to extract line information from registry.");
			continue;
		}

		Add(objLineName, objLineReg.m_sDisplayName, objLineReg.m_sUserName, objLineReg.m_sAuthUser, objLineReg.m_sPassword, objLineReg.m_sRealm, objLineReg.m_sProxy, objLineReg.m_sOutboundProxy, objLineReg.m_sCodec, objLineReg.m_bRegister, objLineReg.m_sInCallType, objLineReg.m_sInCallTo);
	}
}

BOOL CLineList::AddLine(LPCTSTR sLineName, LPCTSTR sDisplayName, LPCTSTR sUserName, LPCTSTR sAuthUser, LPCTSTR sAuthPwd, LPCTSTR sRealm, LPCTSTR sProxy, LPCTSTR sOutboundProxy, LPCTSTR sCodec, BOOL bRegister, LPCTSTR sInCallType, LPCTSTR sInCallTo)
{
	CLineReg objLineReg;
	
	objLineReg.m_sDisplayName	= sDisplayName;
	objLineReg.m_sUserName		= sUserName;
	objLineReg.m_sAuthUser		= sAuthUser;
	objLineReg.m_sPassword		= sAuthPwd;
	objLineReg.m_sRealm			= sRealm;
	objLineReg.m_sProxy			= sProxy;
	objLineReg.m_sOutboundProxy = sOutboundProxy;
	objLineReg.m_sCodec			= sCodec;

	objLineReg.m_bRegister      = bRegister;
		
	objLineReg.m_sInCallType    = sInCallType;
	objLineReg.m_sInCallTo      = sInCallTo;
		
	CRegistry objReg;
	
	if(!objReg.SetLine(sLineName, &objLineReg))
		return FALSE;
		
	Add(sLineName, sDisplayName, sUserName, sAuthUser, sAuthPwd, sRealm, sProxy, sOutboundProxy, sCodec, bRegister, sInCallType, sInCallTo);

	return TRUE;

}

BOOL CLineList::RemoveLine(LPCTSTR sLineName)
{
	CRegistry objReg;
	
	if(!objReg.RemoveLine(sLineName))
		return FALSE;
	
	Remove(sLineName);

	return TRUE;
}

void CLineList::Add(CString sLineName, LPCTSTR sDisplayName, LPCTSTR sUserName, LPCTSTR sAuthUser, LPCTSTR sAuthPwd, LPCTSTR sRealm, LPCTSTR sProxy, LPCTSTR sOutboundProxy, LPCTSTR sCodec, BOOL bRegister, LPCTSTR sInCallType, LPCTSTR sInCallTo)
{
	Remove(sLineName);
	
	CLineData* pLineData = new CLineData;
	
	pLineData->m_dwCallType = atol(sInCallType);
	pLineData->m_sCallTo    = sInCallTo;
	pLineData->m_bRegister  = bRegister;
	
	pLineData->m_sLineName  = sLineName;
	pLineData->m_sCodecList = sCodec;
	
	LONG lLineId = ChooseLineId();
	pLineData->m_lLineId = lLineId; 

	m_pLineListINT->SetAt(lLineId, pLineData);
	
	sLineName.MakeLower();
	m_pLineListTEXT->SetAt(sLineName, pLineData);

	if(!CServerSIP::m_pServerSIP->AddLine(lLineId, sUserName, sAuthUser, sAuthPwd, sDisplayName, sRealm, sProxy, sOutboundProxy))
	{
		CServerSIP::m_pServerSIP->LogError("");
		return;
	}
			
	if(bRegister)
	{
		if(!CServerSIP::m_pServerSIP->RegisterLine(lLineId))
			CServerSIP::m_pServerSIP->LogError("");
	}
}

void CLineList::Remove(CString sLineName)
{
	sLineName.MakeLower();
	
	CLineData* pLineData = FindLine(sLineName);
	if(pLineData == NULL) return;
		
	m_pLineListINT->RemoveKey(pLineData->m_lLineId);
	m_pLineListTEXT->RemoveKey(sLineName);

	if(pLineData->m_bRegister)
		CServerSIP::m_pServerSIP->UnRegisterLine(pLineData->m_lLineId);
	
	delete pLineData;
}

CLineData* CLineList::FindLine(long lLineId)
{
	CLineData* pLineData;

	if(!m_pLineListINT->Lookup(lLineId, pLineData))
		return NULL;
	
	return pLineData;
}

CLineData* CLineList::FindLine(CString sLineName)
{
	sLineName.MakeLower();

	CLineData* pLineData;
	if(!m_pLineListTEXT->Lookup(sLineName, pLineData))
		return NULL;
	
	return pLineData;
}

/*
void CLineList::DeleteAll()
{
	
	ULONG lKey;
	CLineData* pLineData;

	for(POSITION pos = m_pLineListINT->GetStartPosition(); pos != NULL;)
	{
		m_pLineListINT->GetNextAssoc( pos, lKey, pLineData);
		delete pLineData;
	}

	m_pLineListINT->RemoveAll();
	m_pLineListTEXT->RemoveAll();
	

	CString sKey;
	CLineData* pLineData;
			
	int nTotalCall = m_pLineListTEXT->GetCount();
	
	CString* pLineNameList = new CString[nTotalCall];
	POSITION pos = m_pLineListTEXT->GetStartPosition();

	for(int nCount = 0; pos != NULL; nCount++)
	{
		m_pLineListTEXT->GetNextAssoc(pos, sKey, pLineData);
		pLineNameList[nCount] = sKey;
	}

	for(nCount = 0; nCount < nTotalCall; nCount++)
	{
		Remove(pLineNameList[nCount]);
	}
		
	m_pLineListINT->RemoveAll();
	m_pLineListTEXT->RemoveAll();

	delete[] pLineNameList;
}
*/
